/*
Module : DTime+HebrewHolidays.H
Purpose: Defines the interface for the DTime+ CHebrewHolidays class which defines various methods
         to create holidays in the Hebrew calendar
Created: PJN / 29-05-2006
History: PJN / 30-05-2006. 1. Added support for Rosh HaShanah 2
                           2. Renamed CSimhatTorahFunctor to CCSimhatTorahOutsideIsraelFunctor
                           3. Renamed CPassoverFunctor to CPesachFunctor
                           4. Renamed CEndingOfPassoverFunctor to CShviiPesachFunctor
                           5. Added support for Yom Ha'atzmaut
                           6. Added support for Erev Pesach
                           7. Added support for Ta'anit Bechorim
                           8. Added support for Pesach Shenni
                           9. Added support for Yom Yerushalaiym
                           10. Renamed to CHoshahaRabbaFunctor to CHoshanahRabahFunctor

Copyright (c) 2006 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////// Macros / Defines //////////////////////////////////

#ifndef __DTIMEPLUSHEBREWHOLIDAYS_H__
#define __DTIMEPLUSHEBREWHOLIDAYS_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/////////////////////////// Includes //////////////////////////////////////////

#include "DTime+HebrewCalendar.h"
#include "DTime+GregorianCalendar.h"
#include "DTime+CDate.h"
#ifndef __AFXTEMPL_H__
#pragma message("To avoid this message please put afxtempl.h in your PCH (normally stdafx.h)")
#include <afxtempl.h>
#endif


/////////////////////////// Classes ///////////////////////////////////////////

namespace DTimePlus
{

class CRoshHaShanah1Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Tishrei, 1);
  }
};

class CRoshHaShanah2Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Tishrei, 2);
  }
};

class CYomKippurFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Tishrei, 10);
  }
};

class CSukkotFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Tishrei, 15);
  }
};

class CHoshanahRabahFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Tishrei, 21);
  }
};

class CSheminiAzeretFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Tishrei, 22);
  }
};

class CCSimhatTorahOutsideIsraelFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Tishrei, 23);
  }
};

class CTaanitBechorimFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> taanitBechorim(Year, Nissan, 14);
    if (taanitBechorim.GetDayOfWeek() == Saturday)
      return taanitBechorim - 1;
    else
      return taanitBechorim;


    return CDate<CHebrewCalendar>(Year, Nissan, 14);
  }
};

class CErevPesachFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Nissan, 14);
  }
};

class CPesachFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Nissan, 15);
  }
};

class CShviiPesachFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Nissan, 21);
  }
};

class CShavuotFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Sivan, 6);
  }
};

class CHanukkah1Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Kislev, 25);
  }
};

class CHanukkah2Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CHanukkah1Functor()(Year) + 1;
  }
};

class CHanukkah3Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CHanukkah1Functor()(Year) + 2;
  }
};

class CHanukkah4Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CHanukkah1Functor()(Year) + 3;
  }
};

class CHanukkah5Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CHanukkah1Functor()(Year) + 4;
  }
};

class CHanukkah6Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CHanukkah1Functor()(Year) + 5;
  }
};

class CHanukkah7Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CHanukkah1Functor()(Year) + 6;
  }
};

class CHanukkah8Functor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CHanukkah1Functor()(Year) + 7;
  }
};

class CTuBShevatFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Shevat, 15);
  }
};

class CPurimFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, CHebrewCalendar::LastMonthOfYear(CHebrewCalendar::IsLeap(Year)), 14);
  }
};

class CTzomGedaliahFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> tzomGedaliah(Year, Tishrei, 3);
    if (tzomGedaliah.GetDayOfWeek() == Saturday)
      return tzomGedaliah + 1;
    else
      return tzomGedaliah;
  }
};

class CTzomTevetFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> tzomTevet(Year, Tevet, 10);
    if (tzomTevet.GetDayOfWeek() == Saturday)
      return tzomTevet + 1;
    else
      return tzomTevet;
  }
};

class CTaanitEstherFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> purim(CPurimFunctor()(Year));
    if (purim.GetDayOfWeek() == Sunday)
      return purim - 3;
    else
      return purim - 1;
  }
};

class CTzomTammuzFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> tzomTammuz(Year, Tammuz, 17);
    if (tzomTammuz.GetDayOfWeek() == Saturday)
      return tzomTammuz + 1;
    else
      return tzomTammuz;
  }
};

class CTishahBeAvFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> tishahBeAv(Year, Av, 9);
    if (tishahBeAv.GetDayOfWeek() == Saturday)
      return tishahBeAv + 1;
    else
      return tishahBeAv;
  }
};

class CShushanPurimFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> shushanPurim(CPurimFunctor()(Year));
    if (shushanPurim.GetDayOfWeek() == Saturday)
      return shushanPurim + 1;
    else
      return shushanPurim;
  }
};

class CYomHaShoahFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> yomHaShoah(Year, Nissan, 27);
    if (yomHaShoah.GetDayOfWeek() == Sunday)
      return yomHaShoah + 1;
    else
      return yomHaShoah;
  }
};

class CYomHaZikkaronFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    CDate<CHebrewCalendar> yomHaZikkaron(Year, Iyyar, 4);
    DayOfWeek dow = yomHaZikkaron.GetDayOfWeek();
    if (dow == Thursday)
      return yomHaZikkaron - 1; //Move to the prior Wednesday
    else if (dow == Friday)
      return yomHaZikkaron - 2; //Move to the prior Wednesday
    else
      return yomHaZikkaron;
  }
};

class CYomHaatzmautFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CYomHaZikkaronFunctor()(Year) + 1;
  }
};

class CPesachShenniFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Iyyar, 14);
  }
};

class CYomYerushalaiymFunctor
{
public:
  CDate<CHebrewCalendar> operator()(long Year) 
  {
    return CDate<CHebrewCalendar>(Year, Iyyar, 28);
  }
};

//Function which returns all occurences of a specific date in the Hebrew calendar (generated by a functor)
//in a specified Gregorian year. Note that because we are using the flexibility of functors, this function 
//is reusable where we want to calculate Hebrew holidays which do not occur on simple fixed days in the 
//Hebrew calendar
template <class TDATEFUNCTOR>
static void HewbrewOccurencesInGregorianYear(long Year, CArray<CDate<CGregorianCalendar>, CDate<CGregorianCalendar>& >& Occurences)
{
  //Empty out the array before we add any values to it
  Occurences.SetSize(0);

  CDate<CGregorianCalendar> jan1(Year, January, 1);
  long jan1value = jan1.GetValue();
  CDate<CGregorianCalendar> dec31(Year, December, 31);
  long dec31value = dec31.GetValue();
  long y = CDate<CHebrewCalendar>(jan1value, EPOCH_GREG).GetYear();

  TDATEFUNCTOR tdate;
  CDate<CHebrewCalendar> date = tdate(y);
  long datevalue = date.GetValue();
  if (jan1value <= datevalue && datevalue <= dec31value)
  {
    CDate<CGregorianCalendar> gregdate(datevalue, EPOCH_GREG);
    Occurences.Add(gregdate);
  }

  date = tdate(y+1);
  datevalue = date.GetValue();
  if (jan1value <= datevalue && datevalue <= dec31value)
  {
    CDate<CGregorianCalendar> gregdate(datevalue, EPOCH_GREG);
    Occurences.Add(gregdate);
  }

  date = tdate(y+2);
  datevalue = date.GetValue();
  if (jan1value <= datevalue && datevalue <= dec31value)
  {
    CDate<CGregorianCalendar> gregdate(datevalue, EPOCH_GREG);
    Occurences.Add(gregdate);
  }
};

class CHebrewHolidays
{
public:
  inline static bool Omer(const CDate<CHebrewCalendar> date, long& OmerDayNumber) 
  {
    long c = date.GetValue() - CPesachFunctor()(date.GetYear()).GetValue();

    if (1 <= c && c <= 49)
    {
      //Calculate the output parameter
      OmerDayNumber = c;
      return true;
    }
    else
      return false; //specified date is not in Omer
  }
};

} //namespace DTimePlus

#endif //__DTIMEPLUSHEBREWHOLIDAYS_H__
