/*
Module : DTime+MoslemHolidays.H
Purpose: Defines the interface for the DTime+ CMoslemHolidays class which defines various methods
         to create holidays in the Moslem calendar
Created: PJN / 29-05-2006
History: None

Copyright (c) 2006 by PJ Naughter (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


/////////////////////////// Macros / Defines //////////////////////////////////

#ifndef __DTIMEPLUSMOSLEMHOLIDAYS_H__
#define __DTIMEPLUSMOSLEMHOLIDAYS_H__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/////////////////////////// Includes //////////////////////////////////////////

#include "DTime+MoslemCalendar.h"
#include "DTime+GregorianCalendar.h"
#include "DTime+CDate.h"
#ifndef __AFXTEMPL_H__
#pragma message("To avoid this message please put afxtempl.h in your PCH (normally stdafx.h)")
#include <afxtempl.h>
#endif


/////////////////////////// Classes ///////////////////////////////////////////

namespace DTimePlus
{

template <class TCALENDAR>
class CMoslemNewYearFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Muharram, 1);
  }
};

template <class TCALENDAR>
class CAshuraFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Muharram, 10);
  }
};

template <class TCALENDAR>
class CNawlidAnNabiFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, RabiI, 12);
  }
};

template <class TCALENDAR>
class CLailatAlMiRajFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Rajab, 27);
  }
};

template <class TCALENDAR>
class CLailatAlBaraaFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Sha_ban, 15);
  }
};

template <class TCALENDAR>
class CRamdadanFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Ramadan, 1);
  }
};

template <class TCALENDAR>
class CLailatAlKadrFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Ramadan, 27);
  }
};

template <class TCALENDAR>
class CIdAlFitrFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Shawwal, 1);
  }
};

template <class TCALENDAR>
class CIdAlAdhaFunctor
{
public:
  CDate<TCALENDAR> operator()(long Year) 
  {
    return CDate<TCALENDAR>(Year, Dhu_l_Hijja, 10);
  }
};

//Function which returns all occurences of a specific date in the Moslem calendar (generated by a functor)
//in a specified Gregorian year. Note that because we are using the flexibility of functors, this function 
//is reusable if we ever had the requirement to calculate Moslem holidays which do not occur on simple 
//fixed days in the Moslem calendar
template <class TDATEFUNCTOR, class TCALENDAR>
inline static void MoslemOccurencesInGregorianYear(long Year, CArray<CDate<CGregorianCalendar>, CDate<CGregorianCalendar>& >& Occurences)
{
  //Empty out the array before we add any values to it
  Occurences.SetSize(0);

  CDate<CGregorianCalendar> jan1(Year, January, 1);
  long jan1value = jan1.GetValue();
  CDate<CGregorianCalendar> dec31(Year, December, 31);
  long dec31value = dec31.GetValue();
  long y = CDate<TCALENDAR>(jan1value, EPOCH_GREG).GetYear();

  TDATEFUNCTOR tdate;
  CDate<TCALENDAR> date = tdate(y);
  long datevalue = date.GetValue();
  if (jan1value <= datevalue && datevalue <= dec31value)
  {
    CDate<CGregorianCalendar> gregdate(datevalue, EPOCH_GREG);
    Occurences.Add(gregdate);
  }

  date = tdate(y+1);
  datevalue = date.GetValue();
  if (jan1value <= datevalue && datevalue <= dec31value)
  {
    CDate<CGregorianCalendar> gregdate(datevalue, EPOCH_GREG);
    Occurences.Add(gregdate);
  }

  date = tdate(y+2);
  datevalue = date.GetValue();
  if (jan1value <= datevalue && datevalue <= dec31value)
  {
    CDate<CGregorianCalendar> gregdate(datevalue, EPOCH_GREG);
    Occurences.Add(gregdate);
  }
}

} //namespace DTimePlus

#endif //__DTIMEPLUSMOSLEMHOLIDAYS_H__
